
' ==============================================================================

' ChessSet.bas
' Animated chess pieces.
' FreeBASIC Contest #8
' FreeBASIC 0.91.0

' ==============================================================================

' TImage

#define IMG_WIDTH 40
#define IMG_HEIGHT 40
#define RES_PATH ".\bitmap\"

type TImage
  as any ptr bg, fg ' background, foreground
  as integer fX, fY ' screen-relative coordinates
  declare sub Create(byval aFilename as string)
  declare sub Reload(byval aFilename as string)
  declare sub Free
  declare sub Show(byval aX as integer, byval aY as integer)
  declare sub Hide
end type

sub TImage.Create(byval aFilename as string)
  bg = imagecreate(IMG_WIDTH, IMG_HEIGHT)
  fg = imagecreate(IMG_WIDTH, IMG_HEIGHT)
  bload RES_PATH & aFilename, fg
  fX = -1
  fY = -1
end sub

sub TImage.Reload(byval aFilename as string)
  bload RES_PATH & aFilename, fg
end sub

sub TImage.Free
  imagedestroy bg
  imagedestroy fg
end sub

sub TImage.Show(byval aX as integer, byval aY as integer)
  screenlock
  if fX <> -1 then
    put (fX, fY), bg, pset
  end if
  fX = aX
  fY = aY
  get (fX, fY)-(fX + IMG_WIDTH - 1, fY + IMG_HEIGHT - 1), bg
  put (fX, fY), fg, trans
  screenunlock
end sub

sub TImage.Hide
  put (fX, fY), bg, pset
  fX = -1
  fY = -1
end sub

' ==============================================================================

' TChessImage

#define STEP_WIDTH 5
#define DELAY 10
#define X_ORG 40
#define Y_ORG 40
#define UPSIDE_DOWN
#undef UPSIDE_DOWN
#ifdef UPSIDE_DOWN
#define SCR_X(x) IMG_WIDTH * (8 - (x)) + X_ORG
#define SCR_Y(y) IMG_HEIGHT * ((y) - 1) + Y_ORG
#else
#define SCR_X(x) IMG_WIDTH * ((x) - 1) + X_ORG
#define SCR_Y(y) IMG_HEIGHT * (8 - (y)) + Y_ORG
#endif

type TChessImage
  as TImage img
  as integer fX, fY ' chessboard-relative coordinates
  declare sub Create(byval aWhat as string)
  declare sub Free
  declare sub Show overload(byval aX as integer, byval aY as integer)
  declare sub Show overload(byval aCoord as string)
  declare function MoveTo(byval aX as integer, byval aY as integer) as integer
end type

sub TChessImage.Create(byval aWhat as string)
  img.Create(aWhat & ".bmp")
  fX = -1
  fY = -1
end sub

sub TChessImage.Free
  img.Free
end sub

sub TChessImage.Show overload(byval aX as integer, byval aY as integer)
  fX = aX
  fY = aY
  img.Show(SCR_X(fX), SCR_Y(fY))
end sub

sub TChessImage.Show overload(byval aCoord as string)
  dim as integer bx, by
  bx = aCoord[0] - asc("a") + 1
  by = aCoord[1] - asc("1") + 1
  Show(bx, by)
end sub

function TChessImage.MoveTo(byval aX as integer, byval aY as integer) as integer
  dim as integer dx, dy, sx, sy, res = 1
  
  dx = (aX - fX)
  dy = (aY - fY)
  
  if (dx * dy = 0) _
  or (abs(dx) = abs(dy)) _
  or (abs(dx * dy) = 2) then
    
    res = 0
    
    if dx <> 0 then dx \= abs(aX - fX)
    if dy <> 0 then dy \= abs(aY - fY)
    
    if abs(aX - fX) = 2 * abs(aY - fY) then dx *= 2
    if abs(aY - fY) = 2 * abs(aX - fX) then dy *= 2
    
#ifdef UPSIDE_DOWN
    dx *= -1
    dy *= -1
#endif

    sx = SCR_X(fX)
    sy = SCR_Y(fY)
    
    fX = aX
    fY = aY
    
    do
      sx += dx * STEP_WIDTH
      sy -= dy * STEP_WIDTH
      
      sleep DELAY
      
      img.Show(sx, sy)
    loop until (sx = SCR_X(fX)) and (sy = SCR_Y(fY))
  
  end if
  
  return res
end function

' ==============================================================================

' TChessSet

type TChessSet
  dim board as any ptr
  dim pieces(31) as TChessImage
  dim what(31) as string * 2 = { _
    "wk", "wq", _
    "wr", "wr", _
    "wb", "wb", _
    "wn", "wn", _
    "wp", "wp", "wp", "wp", _
    "wp", "wp", "wp", "wp", _
    "bk", "bq", _
    "br", "br", _
    "bb", "bb", _
    "bn", "bn", _
    "bp", "bp", "bp", "bp", _
    "bp", "bp", "bp", "bp" _
  }
  dim where(31) as string * 2 = { _
    "e1", "d1", _
    "a1", "h1", _
    "c1", "f1", _
    "b1", "g1", _
    "a2", "b2", "c2", "d2", _
    "e2", "f2", "g2", "h2", _
    "e8", "d8", _
    "a8", "h8", _
    "c8", "f8", _
    "b8", "g8", _
    "a7", "b7", "c7", "d7", _
    "e7", "f7", "g7", "h7" _
  }
  declare sub Create
  declare sub Free
  declare function MovePiece overload( _
    byval aX1 as integer, _
    byval aY1 as integer, _
    byval aX2 as integer, _
    byval aY2 as integer _
  ) as integer
  declare function MovePiece overload(byval move as string) as integer
  declare function HidePiece overload( _
    byval aX1 as integer, _
    byval aY1 as integer _
  ) as integer
  declare function HidePiece overload(byval square as string) as integer
end type

sub TChessSet.Create
  board = imagecreate(400, 400)
  bload RES_PATH & "board.bmp", board
  put (0, 0), board, trans
  imagedestroy board
  
  for i as integer = 0 to 31
    pieces(i).Create(what(i))
    pieces(i).Show(where(i))
  next i
end sub

sub TChessSet.Free
  for i as integer = 0 to 31
    pieces(i).Free
  next i
end sub

function TChessSet.MovePiece overload( _
  byval aX1 as integer, _
  byval aY1 as integer, _
  byval aX2 as integer, _
  byval aY2 as integer _
) as integer
  dim as integer i, found = 0, res = 1
  
  i = 0
  while (i <= 31) and (found = 0)
    if (pieces(i).fX = aX2) and (pieces(i).fY = aY2) then
      found = 1
      pieces(i).img.Hide
    else
      i += 1
    end if
  wend  
  
  i = 0
  while (i <= 31) and (res = 1)
    if (pieces(i).fX = aX1) and (pieces(i).fY = aY1) then
      res = 0
    else
      i += 1
    end if
  wend
  if res = 0 then
    res = pieces(i).MoveTo(aX2, aY2)
  end if
  return res
end function

function TChessSet.MovePiece overload(byval move as string) as integer
  move = lcase(move)
  return MovePiece( _
    move[0] - asc("a") + 1, _
    move[1] - asc("1") + 1, _
    move[2] - asc("a") + 1, _
    move[3] - asc("1") + 1 _
  )
end function

function TChessSet.HidePiece overload( _
  byval aX1 as integer, _
  byval aY1 as integer _
) as integer
  dim as integer i, bx, by, res = 1
  i = 0
  while (i <= 31) and (res = 1)
    if (pieces(i).fX = aX1) and (pieces(i).fY = aY1) then
      res = 0
    else
      i += 1
    end if
  wend
  if res = 0 then
    pieces(i).img.Hide
  end if
  return res
end function

function TChessSet.HidePiece overload(byval square as string) as integer
  square = lcase(square)
  return HidePiece( _
    square[0] - asc("a") + 1, _
    square[1] - asc("1") + 1 _
  )
end function

' ==============================================================================

' ChessCoord
' Converts screen-relative coordinates into chessboard-relative coordinates.

function ChessCoord(byval x as integer, byval y as integer) as string
  x -= 40
  y -= 40
  x \= 40
  y \= 40
#ifdef UPSIDE_DOWN
  x = 8 - x
  y += 1
#else
  x += 1
  y = 8 - y
#endif
  return chr(asc("a") + x - 1) & chr (asc("1") + y - 1)
end function

' ==============================================================================

' TClickInfo
' Mouse events

enum TClickStatus
  csWaitForButtonDown
  csWaitForButtonRelease
  csValidated
end enum

type TClickInfo
  as string coord
  as TClickStatus status
end type

' ==============================================================================

' Main program

dim set as TChessSet
dim as any ptr board
dim as integer x, y, buttons, gmresult 
dim as TClickInfo click
dim as string cmd, aux

screenres 400, 400, 32
color 0, &hFFFFFF
cls

set.Create

click.status = csWaitForButtonDown
cmd = ""

#ifdef CONSOLE
  shell "echo " & "FreeBASIC Contest #8."
  shell "echo " & "Chess set by Roland Chastain."
  shell "echo " & "Compiled with FreeBASIC version " & __FB_VERSION__ &"."
#endif

do
  gmresult = getmouse (x, y,, buttons)
  
  if gmresult = 0 then
    select case as const click.status
      case csWaitForButtonDown
        if (buttons and 1) = 1 then
          click.coord = ChessCoord(x, y)
          click.status = csWaitForButtonRelease
        end if
      case csWaitForButtonRelease
        if buttons = 0 then
          if ChessCoord(x, y) = click.coord then
            click.status = csValidated
          else
            click.status = csWaitForButtonDown
          end if
        end if
      case csValidated
#ifdef CONSOLE
        shell "echo " & click.coord
#endif
        cmd &= click.coord
        if len(cmd) = 4 then
#ifdef CONSOLE
          shell "echo " & cmd
#endif
          aux = mid(cmd, 1, 2)
          if aux = mid(cmd, 3, 2) then
            set.HidePiece(aux)
          else
            set.MovePiece(cmd)
          end if
          cmd = ""
        end if
        click.status = csWaitForButtonDown
    end select
  end if
  
  sleep 10
loop while inkey = ""

set.Free

end

' Roland Chastain, 2014.

' ==============================================================================
